@@ -3,6 +3,7 @@ from functools import partial, wraps
 from sys import version_info, exc_info
 from threading import RLock
 from types import TracebackType
+from weakref import WeakKeyDictionary
 
 from six import reraise  # type: ignore
 from .async_ import Async
@@ -34,6 +35,7 @@ if False:
         Union,
         Generic,
         Hashable,
+        MutableMapping,
     )
 
 
@@ -319,7 +321,7 @@ class Promise(Generic[T]):
                 if async_guaranteed:
                     promise._is_async_guaranteed = True  # type: ignore
                 self._settle_promise_from_handler(  # type: ignore
-                    handler, value, promise
+                    handler, value, promise  # type: ignore
                 )  # type: ignore
         elif is_promise:
             if async_guaranteed:
@@ -567,7 +569,7 @@ class Promise(Generic[T]):
         did_reject=None,  # type: Optional[Callable[[Exception], S]]
     ):
         # type: (...) -> Promise[S]
-        promise = self.__class__()
+        promise = self.__class__()  # type: Promise
         target = self._target()
 
         state = target._state
@@ -791,7 +793,7 @@ class Promise(Generic[T]):
         dict_type = type(m)  # type: Type[Dict]
 
         if not m:
-            return cls.resolve(dict_type())
+            return cls.resolve(dict_type())  # type: ignore
 
         def handle_success(resolved_values):
             # type: (List[S]) -> Dict[Hashable, S]
@@ -817,7 +819,7 @@ class Promise(Generic[T]):
         )
 
 
-_type_done_callbacks = {}  # type: Dict[type, bool]
+_type_done_callbacks = WeakKeyDictionary()  # type: MutableMapping[type, bool]
 
 
 def is_future_like(_type):
